#include "inkey.ch"
#include "web.ch"
#include "assert.ch"
#INCLUDE "dcdialog.CH"
#INCLUDE "error.CH"
#INCLUDE "fileio.CH"

#Pragma Library("asinet10.lib")

#define PORT 81

#define CRLF Chr(13)+Chr(10)

#define ERR_LOG_FILE      'ERROR.LOG'

#xtranslate NTrim(<n>)   =>  LTrim(Str(<n>))


FUNCTION Main( cHost )

LOCAL cPort, oCxpHttpEndpoint, cHtml, oThread
LOCAL GetList[0], GetOptions, aUrl, i, lStarted := .f.

DEFAULT cHost := "http://" + GetEnv('Computername')
cPort := AllTrim(Str(PORT))

DC_AutoRestoreWindow( { HKEY_CURRENT_USER, 'Software\Donnay\CxpHttpServer\Windows' })

Sleep(10)

oCxpHttpEndpoint := CxpHttpEndpoint():new( PORT )
lStarted := .t.
oCxpHttpEndpoint:start()

oThread := Thread():new()
Sleep(5)
oThread:start({||CxpLoop(oCxpHttpEndpoint)})

@ 0,0 DCSAY "CxpHttpEndpoint sample." FONT '12.Arial Bold'

@ 2,0 DCSAY 'Host: ' + cHost + ':' + cPort SAYSIZE 0 SAYBOTTOM ;
      FONT '12.Lucida Console Bold' COLOR GRA_CLR_BLUE

@ 4,0 DCPUSHBUTTON CAPTION {||IIF(lStarted,'STOP','START') + ' Cxp/Http Server'} SIZE 30,1.5 ;
      FONT '10.Arial Bold' ;
      ACTION {||lStarted := !lStarted, ;
                IIF(lStarted,oCxpHttpEndPoint:start(),oCxpHttpEndPoint:stop()), ;
                DC_GetRefresh(GetList)}

DCGETOPTIONS SAYWIDTH 0

DCREAD GUI FIT TITLE 'Testing CxpHttpEndPoint' ;
  OPTIONS GetOptions

IF lStarted
  oCxpHttpEndpoint:stop()
ENDIF

RETURN nil

* -----------

PROC appsys ; RETURN

* -----------

FUNCTION CxpLoop(oCxpHttpEndpoint)

LOCAL cCxpFile, aDir, cDateTime, cPath, cFileName, cDll, aDir2, ;
      cDateTime2, i, aCxpDir

DO WHILE .t.

  aCxpDir := Directory('*.cxp')
  FOR i := 1 TO Len(aCxpDir)
    cCxpFile := DC_CurPath() + '\' + aCxpDir[i,1]
    aDir := Directory(cCxpFile)
    cDateTime := DtoS(aDir[1,3]) + aDir[1,4]
    cPath := DC_Path(cCxpFile)
    cFileName := DC_Path(cCxpFile,.t.)
    cDll := cPath + 'cxp-application\' + cFileName + '.dll'
    aDir2 := Directory(cDll)
    IF !Empty(aDir2)
      cDateTime2 := DtoS(aDir2[1,3]) + aDir2[1,4]
      IF  cDateTime > cDateTime2 .AND. DllInfo(cDll) > 0
        oCxpHttpEndpoint:stop()
        RunShell('',AppName(.t.),.t.,.t.)
        QUIT
      ENDIF
    ENDIF

  NEXT

  Sleep(50)

ENDDO

RETURN nil

/// <summary>
/// We override DefaultProcessor so we can add our
/// CXP and HTMLFile processors which we need for
/// web debuging.
/// </summary>
CLASS CxpHttpEndPoint FROM HttpEndPoint
  EXPORTED:
  METHOD DefaultProcessors()
  METHOD DefaultConfig()
ENDCLASS

METHOD CxpHttpEndPoint:DefaultProcessors()
  LOCAL aRet := SUPER:DefaultProcessors()
  AAdd( aRet , "MyCxpProcessor" )
  AAdd( aRet , "HtmlFileProcessor" )
RETURN(aRet)

METHOD CxpHttpEndPoint:DefaultConfig(cToken)
  LOCAL oDO := DataObject():New()

  IF( !("CDXDBE" $ dbelist()))
    DbeLoad("CDXDBE")
  ENDIF
  IF( !("FOXDBE" $ dbelist()))
    DbeLoad("FOXDBE")
  ENDIF
  IF( !("FOXCDX" $ dbelist()))
    DbeBuild("FOXCDX","FOXDBE","CDXDBE")
  ENDIF

  cToken := Lower(AllTrim(cToken))

  IF(cToken=="session")
    oDO:Provider := "CxpCookieSessionManager"
    oDO:Timeout  := 15
  ELSEIF(cToken=="storage")
   oDO:Provider := "AppStorageManager"
   oDO:Store    := "cxp-data\application-storage.dbf"
  ENDIF
RETURN(oDO)

* ------------

CLASS MyCxpProcessor FROM CxpProcessor

EXPORTED:

INLINE CLASS METHOD Run( cCxpFile, oWebAppHost, oCxpApplication, ;
                         oHttpRequest, oHttpResponse )

LOCAL x, oError

BEGIN SEQUENCE

  x := SUPER:run( cCxpFile, oWebAppHost, oCxpApplication, ;
                  oHttpRequest, oHttpResponse )

RECOVER USING oError

  oHttpResponse:contentType := 'text/html'
  oHttpResponse:writeStr( oError:cargo )
  oHttpResponse:send()

END SEQUENCE

RETURN x

ENDCLASS


* ------------

CLASS CustomWebHandler FROM WebHandler

EXPORTED:

INLINE METHOD GetVar( cVarName )
RETURN ::httpRequest:getForm():getVariable(cVarName)

INLINE METHOD SetErrorBlock()
ErrorBlock({|e|LogError(e)})
RETURN self

// Dispatch the request to existing methods. Return a generic
// web page for all other requests.
//
INLINE METHOD Execute( cMethodName )

LOCAL cResponse := ""

::setErrorBlock()

IF IsMethod(SELF, cMethodName)
  RETURN SUPER:execute( cMethodName )
ENDIF

cResponse += "<!DOCTYPE html>"
cResponse += "<html>"
cResponse +=   "<head><title>Unhandled request</title></head>"
cResponse +=   "<body>"
cResponse +=     "<h1>No handler for: "+cMethodName+"</h1>"
cResponse +=     "<small>Sorry</small>"
cResponse +=   "</body>"
cResponse += "<html>"

RETURN cResponse

ENDCLASS

* ------------

STATIC FUNCTION LogError( oError )

LOCAL cError, i, cProcName

cError := "= Donnay Software Web Server Error ==========" + CRLF +;
             "Date: " + DTOS(Date()) + ", Time: " + Time() + CRLF +;
             "App: " + AppName(.t.) + ", Thread: " + LTrim(Str(ThreadID())) + CRLF

// error generated while processing http server request
cError += if(oError:severity > ES_WARNING, "ERROR ", "WARNING " ) +;
          if(ValType(oError:subSystem)   == "C", oError:subSystem, "???" ) + "/" +;
          if(ValType(oError:subCode)     == "N", NTrim(oError:subCode), "???" ) + CRLF +;
          if(ValType(oError:cargo)       == "C", oError:cargo + CRLF, "" ) +;
          if(ValType(oError:description) == "C", oError:description, "" )

// add either filename or operation
if !Empty(oError:filename)
   cError += ": " + oError:filename
elseif !Empty(oError:operation)
   cError += ": " + oError:operation
endif

if !Empty( oError:osCode )
   cError += CRLF + "(OS Error: " + NTrim(oError:osCode) + ")"
endif

for i := 2 to 12
   if Empty(cProcName := ProcName(i))
      exit
   endif
   cError += CRLF+"Called from "+cProcName+"("+LTrim(Str(ProcLine(i)))+")"
next

SaveToFile( cError + CRLF, ERR_LOG_FILE )

cError := '<pre>' + cError + '</pre>'

Break( cError )

RETURN ''

* -----------

STATIC FUNCTION SaveToFile( cText, cFileName )

Local i := 0, nHandle

if ValType(cText) != "C" .or. Empty(cText)
   Return .f.
endif

if FExists(cFileName)
   while (nHandle := FOpen(cFileName, FO_WRITE + FO_DENYWRITE)) < 0
      if ++i > 9
         Return .f.
      endif
      Sleep(20)
   end
   FSeek(nHandle, 0, FS_END)  // position to end of file
else
   while (nHandle := FCreate(cFileName)) < 0
      if ++i > 9
         Return .f.
      endif
      Sleep(20)
   end
endif

FWrite(nHandle, cText + CRLF)
FClose(nHandle)

Return .t.

* -----------

PROCEDURE DBESYS
  DbeLoad("FOXDBE")
  DbeLoad("CDXDBE")
  DbeBuild("FOXCDX","FOXDBE","CDXDBE")
RETURN


