#include "dcdialog.ch"

#xtranslate NTrim(<n>)   =>  LTrim(Str(<n>))

#define CRLF  Chr(13)+Chr(10)

CLASS DD787 FROM CustomWebHandler

  EXPORTED:
    METHOD Rsvp
    CLASS METHOD PlaceRsvp
    CLASS METHOD ConfirmRsvp
ENDCLASS

* ---------

METHOD DD787:rsvp

LOCAL cHtml, oForm, cAction, cErrorMessage, lSendForm, i, ;
      cValue, cReplace, lTimeOut, nTimeStamp

::setErrorBlock()
BEGIN SEQUENCE

oForm := RSVPForm():new()

oForm:getAllVars(::httpRequest:form)

cAction := ::httpRequest:queryString

cErrorMessage := ''

nTimeStamp := Val(::httpRequest:form:timestamp)

lTimeout := nTimeStamp > 0 .AND. Seconds() - nTimeStamp > 300 // Session timeout after 5 minutes

lSendForm := oForm:ValidateVars(@cErrorMessage,lTimeOut)

IF lSendForm

  TEXT INTO cHtml
  <html>
  <head>
  <title>USS James E. Kyes Reunion RSVP</title>
  <link rel="stylesheet" type="text/css" href="../donnay.css" />
  </head>
  <body bgcolor="#FFFFFF" background="./backgrounds/grey.jpg">
  ENDTEXT

  IF lTimeOut
    oForm:getAllVars() // Empty out all vars
    cErrorMessage := 'Your session timed out! Please re-enter all data!'
  ENDIF

  cHtml += oForm:stuffVars(cErrorMessage) // replace html text with validated data

  cHtml := Strtran(cHtml,'@formtag@','<form METHOD=get ACTION="./rsvp?PlaceRSVP">')

  cHtml += '</body></html>'

ELSEIF 'ConfirmRSVP' $ cAction

  cHtml := ::ConfirmRSVP(oForm)

ELSEIF 'PlaceRSVP' $ cAction

  cHtml := ::PlaceRSVP(oForm)

ENDIF

RECOVER USING cHtml
END SEQUENCE

RETURN  cHtml

* ---------

CLASS METHOD DD787:PlaceRSVP( oForm )

LOCAL cError, cResponse := '', cText, i, cVarName, cValue

TEXT INTO cText
Please review your RSVP information below.  If everything is correct,
click on the CONFIRM button, otherwise click on the BACK button of your
Web browser and make the necessary changes.

<H4>NOTE: Your RSVP will not be processed until you click on <B>CONFIRM</B></H4>
ENDTEXT

cText := StrTran( cText, '<<NAME>>',oForm:name)
cText := StrTran( cText, '<<EMAIL>>',oForm:email)
cText += oForm:RSVPText( 1 )
cResponse += cText

cResponse += '<FORM ACTION="./RSVP?ConfirmRSVP" METHOD=GET>'
cResponse += '<INPUT TYPE=SUBMIT NAME="ConfirmRSVP" VALUE="Confirm!">'
cResponse += oForm:ReplicateVars( {'PlaceRSVP'} )
cResponse += '</FORM>'

RETURN cResponse

* ----------

CLASS METHOD DD787:ConfirmRSVP( oForm )

LOCAL cText, cHtml := '', cFileName

cText := MemoRead('rsvp787.txt')
cText := StrTran( cText, '<<NAME>>',oForm:name)
cText := StrTran( cText, '<<EMAIL>>',oForm:email)
cText += oForm:RSVPText( 2 )

cFileName := Dtos(Date())+ '_'+Time()+'_Email.txt'
cFileName := Strtran(cFileName,':','')

MemoWrit( cFileName,cText )
// EMAIL response to Client
SendMail( ,, Alltrim(oForm:email), 'USS James E. Kyes Reunion RSVP Confirmation',,, cText )

cText := MemoRead('rsvp787.txt')
cText := StrTran( cText, '<<NAME>>',oForm:name)
cText := StrTran( cText, '<<EMAIL>>',oForm:email)
cText += oForm:RSVPText( 1 ) // EMAIL response to Secretary

// EMAIL response to Secretary
SendMail( ,Alltrim(oForm:email), 'RogerDonnay@Donnay-Software.Com', ;
          'USS James E. Kyes Reunion RSVP Confirmation',,, cText )

cHtml += '<p>An email message has been sent to <b><em>' + Alltrim(oForm:email) + ;
         ' </em></b>with confirmation of your RSVP'

RETURN cHtml

* ==============

CLASS RSVPForm

HIDDEN:
VAR varInfo

EXPORTED:
VAR city
VAR email
VAR name
VAR phone
VAR remarks
VAR state
VAR street1
VAR street2
VAR zip
VAR years
VAR timestamp

INLINE METHOD init
LOCAL i
::varInfo := ::classdescribe()[3]
FOR i := 1 TO Len(::varInfo)
  ::&(::varInfo[i,1]) := ''
  ::varInfo[i,2] := ''
  ::varInfo[i,3] := ''
NEXT

ASort(::varInfo,,,{|a,b|a[1]<b[1]})

RETURN self

* ---------

INLINE METHOD getAllVars(oForm)
LOCAL i, c
::varInfo := ::classdescribe()[3]
FOR i := 1 TO Len(::varInfo)
  ::varInfo[i,1] := Lower(::varInfo[i,1])
  IF Valtype(oForm) == 'O'
    ::varInfo[i,2] := oForm:&(::varInfo[i,1])
  ELSE
    ::varInfo[i,2] := ''
  ENDIF
  ::&(::varInfo[i,1]) := ::varInfo[i,2]
  ::varInfo[i,3] := ''
  IF ::varInfo[i,1] == 'timestamp'
    ::varInfo[i,2] := Alltrim(Str(Seconds()))
  ENDIF
NEXT
ASort(::varInfo,,,{|a,b|a[1]<b[1]})
RETURN ::varInfo

* ---------

INLINE METHOD ValidateVars(cErrorMessage,lTimeOut)

LOCAL i, cVarName, cValue, lError := .f., nQty := 0

FOR i := 1 TO Len(::varInfo)
  cVarName := ::varInfo[i,1]
  cValue := Trim(::varInfo[i,2])
  IF lTimeout .OR. cVarName $ {'name','street1','city','zip','email','phone','state'} .AND. Empty(cValue)
    ::varInfo[i,3] := ' ***'
    lError := .t.
  ELSEIF cVarName = 'email' .AND. !('@'$cValue)
    ::varInfo[i,3] := ' Invalid email address'
    lError := .t.
  ELSE
    ::varInfo[i,3] := ''
  ENDIF
NEXT

IF lError
  cErrorMessage := 'Fields marked by *** are required'
ENDIF

RETURN lError

* -----------

INLINE METHOD ReplicateVars( aExclude )

LOCAL i, cResponse := ''

DEFAULT aExclude := {}

FOR i := 1 TO Len(::varInfo)
  IF ::varInfo[i,1] $ aExclude
    LOOP
  ENDIF
  cResponse += '<input type=hidden name="' + ;
               ::varInfo[i,1] + '" value="' + ;
               ::varInfo[i,2] + '">' + CRLF
NEXT

RETURN cResponse

* -----------

INLINE METHOD StuffVars( cErrorMessage )

LOCAL i, cHtml := MemoRead('rsvp787.htm'), cValue, cReplace

DEFAULT cErrorMessage := ''

FOR i := 1 TO Len(::varInfo)
  cValue := ::varInfo[i,2]
  cReplace := IIF(!Empty(cValue),'value="' + cValue+'"',"") + ' name="' + ::varInfo[i,1] + '"'
  cHtml := Strtran( cHtml, '@' + ::varInfo[i,1] + '@', cReplace )
  cHtml := Strtran( cHtml, '%' + ::varInfo[i,1] + '%', ::varInfo[i,3] )
NEXT

cHtml := Strtran( cHtml, '%errormessage%', cErrorMessage )

RETURN cHtml

* ----------

INLINE METHOD RSVPText( nMode )

LOCAL cText := ''

cText += CRLF
IF nMode = 1
  cText += '<pre>' + CRLF
ENDIF
cText += '            Name: ' + ::name       + CRLF
cText += '       Address 1: ' + ::street1    + CRLF
cText += '       Address 2: ' + ::street2    + CRLF
cText += '            City: ' + ::city       + CRLF
cText += '           State: ' + ::state      + CRLF
cText += '             Zip: ' + ::zip        + CRLF
cText += '           Phone: ' + ::phone      + CRLF
cText += '           Email: ' + ::email      + CRLF
cText += '  Years on Board: ' + ::years      + CRLF

cText += 'Remarks: ' + Strtran(::remarks,CRLF+CRLF,CRLF) + CRLF

RETURN cText

ENDCLASS


