/*
This sample program is a Picture Server that serves up
pictures to either the Picture Client (dcpictclent.prg)
or a Web Browser
*/

#define DEFAULT_PORT 81

#include "Xbp.ch"
#include "Appevent.ch"
#include "web.CH"
#include "assert.ch"
#INCLUDE "dcdialog.CH"

#PRAGMA LIBRARY( "ascom10.lib" )

PROCEDURE Main

LOCAL oPictureServerUI

SET DEFAULT TO ..\..\data\northwind\dbf

oPictureServerUI := PictureServerUI():new()

oPictureServerUI:shutdown()

RETURN

* ---------

PROCEDURE AppSys() ; RETURN

* ---------

PROCEDURE DbeSys()
  IF .NOT. DbeLoad( "FOXDBE", .F. )
    MsgBox( "Can not load FOXDBE" )
    quit
  ENDIF
RETURN

* --------

CLASS PictureServerUI FROM WebServerUI

EXPORTED:

INLINE METHOD Init()

PictureServer(self)
WebSocketPictureServer(self)
::webServerUI:init()
RETURN self

INLINE METHOD ProcessPicture( cPicturePath )
::webServerUI:_processing := cPicturePath
DC_GetRefresh(::_getList)
RETURN self

ENDCLASS

* ==================

CLASS PictureServer FROM WebHandler

PROTECTED:
    CLASS VAR _UI

EXPORTED:

INLINE CLASS METHOD initClass( oUI )
  ::_UI := oUI

RETURN self

//The URL with the RESTful path "http://<servername>:<port>/PictureServer/start"
//returns the start page.
INLINE METHOD start()

LOCAL cHtml, cUrl
cUrl  := "ws://"+::_UI:WSUrl
cHtml := GetHtml()
cHtml := StrTran( cHtml, "<!--[URL]-->", cUrl )

RETURN cHtml

ENDCLASS

* ===============

CLASS WebSocketPictureServer FROM WebSocketServer

PROTECTED:

EXPORTED:

VAR _alias

INLINE METHOD init()

  DbUseArea( .T., "FOXDBE", "employees" )
  ::_alias := Alias()
  ::webSocketServer:init()

  RETURN self

INLINE METHOD _sendCurrentPicture()

  ::sendBinary( (::_alias)->photo )

  ::_UI:processPicture( AllTrim((::_alias)->firstname) + " " + ;
                        AllTrim((::_alias)->lastname) )
RETURN self

INLINE METHOD onText( cText )
  cText := Lower( cText )
  DO CASE
    CASE cText == "current"
      // no navigation here
    CASE cText == "next"
      (::_alias)->(DbSkip())
      IF (::_alias)->(Eof())
        (::_alias)->(DbGoTop())
       ENDIF
    CASE cText == "prev"
      (::_alias)->(DbSkip(-1))
      IF (::_alias)->(Bof())
        (::_alias)->(DbGoBottom())
      ENDIF
    OTHERWISE
      RETURN self
  ENDCASE
  ::_sendCurrentPicture()

RETURN self

ENDCLASS

* ------------

STATIC FUNCTION GetHtml()

LOCAL cHtml

TEXT INTO cHtml WRAP

<!DOCTYPE HTML >

<html>
<head>
   <title>websockets client</title>
   <meta http-equiv="X-UA-Compatible" content="IE=edge" >
   <script type="text/javascript">

      //The url is fed in from the server before the page is
      //delivered to the browser
      var url    = "<!--[URL]-->";
      var socket = null;

      window.onload = function () {
         if( typeof(WebSocket) != "function" ){
            setNoWebSocketSupport();
            return;
         }
      //   window.Initialize();
       }

      function ConnectWebSocket() {
         try {
            socket = new WebSocket(url);

            setServerStatus( 'Closed' );
            setServerError( '' );

            socket.onopen = function (openEvent) {
               setServerStatus( 'Open' );
               setServerError( '' );
               SetButtonConnected( true );
               sendTextCommand('current');
            };

            socket.onclose = function (closeEvent) {
               setServerStatus( 'Closed' );
               SetButtonConnected( false );
            };

            socket.onmessage = function (messageEvent) {
               if (messageEvent.data instanceof Blob) {
                  var image = document.getElementById('image');
                  image.src = window.URL.createObjectURL(messageEvent.data);
               }else{
                  setServerResponse( '<b>'+messageEvent.data+'</b>' );
               }
            };

            socket.onerror = function (errorEvent) {
               SetButtonConnected( false );
               if( errorEvent.data == undefined ) return;
               setServerError( errorEvent.data );
            };

         }
         catch (exception) {
            setServerError( 'Exception occurred' );
            SetButtonConnected( false );
         }
      }
      function DisconnectWebSocket() {
         if (socket == undefined) return;
         socket.close();
         socket = null;
      }
      function sendTextCommand( textCommand ){
         if( socket == undefined ) return;
         if (socket.readyState != WebSocket.OPEN) return;
         socket.send( textCommand );
      }
      function setInnerHtmlById( idElement, text ){
         document.getElementById(idElement).innerHTML = text;
      }
      function setDisabledById( idElement, isDisabled ){
         document.getElementById(idElement).disabled = isDisabled;
      }
      function setNoWebSocketSupport(){
         var cMsg
         cMsg =  "<h1>Error</h1> Your browser does not support websockets.";
         document.getElementById("body").innerHTML = cMsg;
      }
      function setServerStatus( text )   { setInnerHtmlById( "serverStatus",   text ) }
      function setServerResponse( text ) { setInnerHtmlById( "serverResponse", text ) }
      function setServerError( text ){
         if( text != '' ){
            text = 'Error: (' + text + ')';
         }
         setInnerHtmlById( "serverError", text )
      }
      function SetButtonConnected( isConnected ){
         setDisabledById( 'idBtnConnect',      isConnected );
         setDisabledById( 'idBtnDisconnect', ! isConnected );
         setDisabledById( 'idBtnPrev',       ! isConnected );
         setDisabledById( 'idBtnNext',       ! isConnected );
      }
      function Initialize() {
         ConnectWebSocket();
         //SetButtonConnected( true );
      }
      function Deinitialize() {
         DisconnectWebSocket();
         //SetButtonConnected( false );
      }

   </script>
</head>

<body id="body">
    <div>
        Status: (<span id="serverStatus"></span>)
        <span id="serverError"></span>
        <br>
        <button id="idBtnConnect"    onclick="window.Initialize();return true;">Connect</button>
        <button id="idBtnDisconnect" disabled="disabled" onclick="window.Deinitialize();return true;">Disconnect</button>
        <button id="idBtnPrev"       disabled="disabled" onclick="sendTextCommand('prev');return true;">Prev</button>
        <button id="idBtnNext"       disabled="disabled" onclick="sendTextCommand('next');return true;">Next</button><br/>
        <br>
        <div id="serverResponse"></div>
        <p>
        <img id="image"  />
        </p>

    </div>
</body>
</html>

ENDTEXT

RETURN cHtml
