#define DEFAULT_PORT 81

#include "Xbp.ch"
#include "Appevent.ch"
#include "web.CH"
#INCLUDE "dcdialog.CH"
#INCLUDE "assert.CH"

#PRAGMA LIBRARY( "ascom10.lib" )

CLASS WebServerUI

   VAR _HttpEndpoint

EXPORTED:

   VAR _ConnectedClientsNum
   VAR _getList
   VAR _isRunning
   VAR _port
   VAR _computerName
   VAR _processing

   METHOD init
   METHOD shutdown
   METHOD startup
   METHOD go
   METHOD copyUrl

   // methods invoked by the view model
   METHOD connected
   METHOD disconnected

   ACCESS METHOD getUrl()   VAR URL
   ACCESS METHOD getWSUrl() VAR WSURL

ENDCLASS

******************************************************************************
* Initialize and create form
******************************************************************************

METHOD WebServerUI:init()

LOCAL GetList[0], GetOptions

::_getList := GetList
::_port := DEFAULT_PORT
::_connectedClientsNum := 0
::_computerName := GetEnv('Computername')
::_isRunning := .f.
::_processing := ''

@   0,12  DCSAY 'Server control' SAYHVCENTER SIZE 516,24 COLOR GRA_CLR_WHITE, GRA_CLR_BLUE
@  36,24  DCSAY 'Computername:' SAYRIGHTCENTER SIZE 96,24 COLOR nil,-255
@  36,132 DCSAY ::_computerName SAYVCENTER SIZE 228, 24 COLOR nil, -255 FONT '12.Arial'
@  60,36  DCSAY 'Port:' SAYRIGHTCENTER SIZE 84,24 COLOR nil, -255
@  60,132 DCSPINBUTTON ::_port SIZE 60,24 COLOR nil, XBPSYSCLR_ENTRYFIELD WHEN {||!::_isRunning}
@  60,432 DCPUSHBUTTON CAPTION 'Start' SIZE 96,24 COLOR -58, XBPSYSCLR_BUTTONMIDDLE ;
          ACTION {||::startUp()} WHEN {||!::_isRunning}
@  84,36  DCSAY 'Status' SAYRIGHTCENTER SIZE 84,24 COLOR nil, -255
@  84,432 DCPUSHBUTTON CAPTION 'Stop' SIZE 96,24 COLOR -58, XBPSYSCLR_BUTTONMIDDLE ;
          ACTION {||::shutDown()} WHEN {||::_isRunning}
@  84,132 DCSAY {||IIF(::_isRunning,'Running','Stopped')} SAYVCENTER SIZE 84,24 ;
          COLOR {||IIF(::_isRunning,{GRA_CLR_DARKGREEN,-255},{GRA_CLR_RED,-255})}
@ 108,24  DCSAY 'Connected Clients:' SAYRIGHTCENTER SIZE 96,24 COLOR nil, -255
@ 108,132 DCSAY {||::_connectedClientsNum} SAYVCENTER SIZE 84,24 ;
          COLOR {||IIF(::_isRunning,{GRA_CLR_RED,-255},{GRA_CLR_BLACK,-255})}
@ 144,12  DCSAY 'Information' SAYHVCENTER SIZE 516,24 COLOR GRA_CLR_WHITE, GRA_CLR_DARKGRAY
@ 182,432 DCPUSHBUTTON CAPTION 'URL to Clipboard' SIZE 96,24 COLOR -58, XBPSYSCLR_BUTTONMIDDLE ;
          ACTION {||::copyUrl()}
@ 192, 36 DCSAY 'Browse URL:' SAYRIGHTCENTER SIZE 84,24 COLOR nil, -255
@ 192,132 DCSAY {||IIF(::_isRunning,"http://"+::Url ,'')} SAYVCENTER SIZE 246,24 COLOR nil, -255
@ 206,432 DCPUSHBUTTON CAPTION 'Start Client' SIZE 96,24 COLOR -58, XBPSYSCLR_BUTTONMIDDLE ;
          ACTION {||::go(2)} WHEN {||::_isRunning}
@ 230,432 DCPUSHBUTTON CAPTION 'Start Browser' SIZE 96,24 COLOR -58, XBPSYSCLR_BUTTONMIDDLE ;
          ACTION {||::go(3)} WHEN {||::_isRunning}
@ 264,12  DCSAY 'Activity' SAYHVCENTER SIZE 516,24 COLOR GRA_CLR_WHITE, GRA_CLR_BROWN
@ 296,36  DCSAY 'Processing:' SAYRIGHTCENTER SIZE 84,24 COLOR nil, -255
@ 296,132 DCSAY {||::_processing} SAYVCENTER SIZE 396,24 COLOR nil, -255

DCGETOPTIONS PIXEL

DCREAD GUI FIT TITLE 'Web Server' MODAL OPTIONS GetOptions

RETURN self

METHOD WebServerUI:shutdown()
  ::_isRunning := .f.
  DC_GetRefresh(::_getList)
  IF .NOT. (NIL==::_HttpEndpoint)
    ::_HttpEndpoint:stop()
    ::_HttpEndpoint := NIL
  ENDIF
  WebSocketPictureServer():closeAll()
RETURN self

METHOD WebServerUI:startup()
  ::_isRunning := .t.
  DC_GetRefresh(::_getList)
  ::_HttpEndpoint := HttpEndpoint():new(::_port)
  ::_HttpEndpoint:start()
RETURN self

METHOD WebServerUI:go( nWhich )
  LOCAL cUrl, oThread
  IF nWhich == 3
    cUrl := "http://"+::Url
    oThread := Thread():new()
    Sleep(5)
    oThread:start({||RunBrowser(::getUrl())})
  ELSEIF nWhich == 1
    RunShell( "/C explorer.exe "+cUrl,, .T., .T. )
  ELSEIF nWhich == 2
    RunShell( '','dcpictclient.exe',.t.,.t. )
  ENDIF
RETURN self

METHOD WebServerUI:copyUrl()
  LOCAL cUrl, oClipBoard
  cUrl := "http://"+::Url

  oClipBoard := XbpClipBoard():new():create()
  oClipBoard:open()
  oClipBoard:clear()
  oClipBoard:setBuffer( cUrl )
  oClipBoard:close()
  oClipBoard:destroy()
RETURN self

METHOD WebServerUI:connected()
  ::_ConnectedClientsNum++
  DC_GetRefresh(::_getList)
RETURN self

METHOD WebServerUI:disconnected()
  ::_ConnectedClientsNum--
  DC_GetRefresh(::_getList)
RETURN self

METHOD WebServerUI:getUrl()
  LOCAL cUrl := ""
  cUrl += ::_computerName + ":"
  cUrl += AllTrim(Str(::_port))
  cUrl += "/pictureserver/start"
RETURN cUrl

METHOD WebServerUI:getWSUrl()
  LOCAL cUrl := ""
  cUrl += ::_computerName + ":"
  cUrl += AllTrim(Str(::_port))
  cUrl += "/websocketpictureserver"
RETURN cUrl

* ==================

CLASS WebSocketServer FROM WebSocketHandler

  PROTECTED:
    CLASS VAR _UI
    CLASS VAR _Instances

    SYNC CLASS METHOD _addSelf
    SYNC CLASS METHOD _delSelf

  EXPORTED:
         CLASS METHOD initClass
    SYNC CLASS METHOD closeAll

               METHOD init
               METHOD onConnect
               METHOD onDisconnect
               METHOD onBinary
ENDCLASS

CLASS METHOD WebSocketServer:_addSelf( oSelf )
  AAdd( ::_Instances, oSelf )
RETURN self

CLASS METHOD WebSocketServer:_delSelf( oSelf )
  LOCAL nPos
  nPos := AScan( ::_Instances, oSelf )
  ASSERT( nPos != 0 )
  ARemove( ::_Instances, nPos )
RETURN self

CLASS METHOD WebSocketServer:initClass( oUI )
  ::_UI        := oUI
  ::_Instances := {}
RETURN self

CLASS METHOD WebSocketServer:closeAll()
  LOCAL oInstance, nInstance, nInstances
  nInstances := Len( ::_Instances )
  FOR nInstance := 1 to nInstances
    oInstance := ::_Instances[nInstance]
    oInstance:close()
  NEXT
  ::_Instances := {}
RETURN self

METHOD WebSocketServer:init()
  ::_addSelf(self )
RETURN self

METHOD WebSocketServer:onConnect()
  ::_UI:connected()
RETURN self

METHOD WebSocketServer:onDisconnect()
  ::_UI:disconnected()
  dbCloseAll()
RETURN self

METHOD WebSocketServer:onBinary( cBinary )
  Unused( cBinary )
RETURN self

* =======================

STATIC FUNCTION RunBrowser( cUrl )

LOCAL GetList[0], oViewer

cUrl := 'http://' + cUrl

@ 0,0 DCHTMLVIEWER oViewer SIZE 300,300 PIXEL ;
      NAVIGATE cUrl

DCREAD GUI FIT TITLE 'Picture Server Client Browser'

RETURN nil



